@@ -2,71 +2,36 @@
 
 """Resolve classes."""
 
-import collections.abc
 import inspect
 import logging
-from operator import attrgetter
 from textwrap import dedent
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Collection,
-    Dict,
-    Generic,
-    Iterable,
-    Iterator,
-    List,
-    Mapping,
-    Optional,
-    Sequence,
-    Set,
-    Type,
-    TypeVar,
-    Union,
+from typing import Any, Collection, List, Mapping, Optional, Sequence, Type, TypeVar
+
+from .base import BaseResolver
+from .utils import (
+    Hint,
+    HintOrType,
+    HintType,
+    OneOrSequence,
+    OptionalKwargs,
+    get_subclasses,
+    normalize_string,
+    upgrade_to_sequence,
 )
 
-from pkg_resources import iter_entry_points
-
-if TYPE_CHECKING:
-    import click
-
 __all__ = [
     # Type Hints
-    "InstOrType",
-    "Lookup",
-    "LookupType",
-    "LookupOrType",
-    "Hint",
-    "HintType",
-    "HintOrType",
-    "OptionalKwargs",
     # Classes
+    "ClassResolver",
     "Resolver",
     # Utilities
-    "get_subclasses",
     "get_cls",
-    "normalize_string",
     # Exceptions
     "KeywordArgumentError",
     "UnexpectedKeywordError",
 ]
 
 X = TypeVar("X")
-Y = TypeVar("Y")
-
-InstOrType = Union[X, Type[X]]
-
-Lookup = Union[str, X]
-LookupType = Lookup[Type[X]]
-LookupOrType = Lookup[InstOrType[X]]
-
-Hint = Optional[Lookup[X]]
-HintType = Hint[Type[X]]
-HintOrType = Hint[InstOrType[X]]
-
-OptionalKwargs = Optional[Mapping[str, Any]]
-OneOrSequence = Union[X, Sequence[X]]
 
 logger = logging.getLogger(__name__)
 
@@ -107,102 +72,67 @@ MISSING_ARGS = [
 ]
 
 
-class Resolver(Generic[X]):
+class ClassResolver(BaseResolver[Type[X], X]):
     """Resolve from a list of classes."""
 
     #: The base class
     base: Type[X]
     #: The shared suffix fo all classes derived from the base class
     suffix: str
-    #: The mapping from normalized class names to the classes indexed by this resolver
-    lookup_dict: Dict[str, Type[X]]
-    #: The mapping from synonyms to the classes indexed by this resolver
-    synonyms: Dict[str, Type[X]]
     #: The variable name to look up synonyms in classes that are registered with this resolver
     synonyms_attribute: Optional[str]
 
     def __init__(
         self,
-        classes: Collection[Type[X]],
+        classes: Optional[Collection[Type[X]]] = None,
         *,
         base: Type[X],
         default: Optional[Type[X]] = None,
         suffix: Optional[str] = None,
         synonyms: Optional[Mapping[str, Type[X]]] = None,
         synonym_attribute: Optional[str] = "synonyms",
+        base_as_suffix: bool = True,
     ) -> None:
         """Initialize the resolver.
 
         :param classes: A list of classes
         :param base: The base class
         :param default: The default class
-        :param suffix: The optional shared suffix of all classes. If None, use the base class' name for it. To disable
-            this behaviour, explicitly provide `suffix=""`.
+        :param suffix: The optional shared suffix of all instances. If not none, will override
+            ``base_as_suffix``.
         :param synonyms: The optional synonym dictionary
         :param synonym_attribute: The attribute to look in each class for synonyms. Explicitly set to None
             to turn off synonym lookup.
+        :param base_as_suffix: Should the base class's name be used as the suffix if none is given? Defaults to true.
         """
         self.base = base
-        self.default = default
-        if suffix is None:
-            suffix = normalize_string(base.__name__)
-        self.suffix = suffix
         self.synonyms_attribute = synonym_attribute
-        self.synonyms = dict(synonyms or {})
-        self.lookup_dict = {}
-        for cls in classes:
-            self.register(cls)
+        if suffix is not None:
+            if suffix == "":
+                suffix = None
+        elif base_as_suffix:
+            suffix = normalize_string(self.base.__name__)
+        super().__init__(
+            elements=classes,
+            synonyms=synonyms,
+            default=default,
+            suffix=suffix,
+        )
 
-    def register(
-        self,
-        cls: Type[X],
-        synonyms: Optional[Iterable[str]] = None,
-        raise_on_conflict: bool = True,
-    ) -> None:
-        """Register an additional class with this resolver.
+    def extract_name(self, element: Type[X]) -> str:
+        """Get the name for an element."""
+        return element.__name__
 
-        :param cls: The class to register
-        :param synonyms: An optional iterable of synonyms to add for the class
-        :param raise_on_conflict: Determines the behavior when a conflict is encountered on either
-            the normalized class name or a synonym. If true, will raise an exception. If false, will
-            simply disregard the entry.
-
-        :raises KeyError: If ``raise_on_conflict`` is true and there's a conflict in either the class
-            name or a synonym name.
-        :raises ValueError: If any given synonyms (either explicitly or by class lookup) are empty strings
-        """
-        key = self.normalize_cls(cls)
-        if key not in self.lookup_dict:
-            self.lookup_dict[key] = cls
-        elif raise_on_conflict:
-            raise KeyError(
-                f"This resolver already contains a class with key {key}: {self.lookup_dict[key]}"
-            )
-
-        _synonyms = set(synonyms or [])
-        if self.synonyms_attribute is not None:
-            _synonyms.update(getattr(cls, self.synonyms_attribute, None) or [])
-
-        self.lookup_dict[key] = cls
-        for synonym in _synonyms:
-            synonym_key = self.normalize(synonym)
-            if not synonym_key:
-                raise ValueError(f"Tried to use empty synonym for {cls}")
-            if synonym_key not in self.synonyms and synonym not in self.lookup_dict:
-                self.synonyms[synonym_key] = cls
-            elif raise_on_conflict:
-                raise KeyError(
-                    f"This resolver already contains synonym {synonym} for {self.synonyms[synonym]}"
-                )
-
-    def __iter__(self) -> Iterator[Type[X]]:
-        """Return an iterator over the indexed classes sorted by name."""
-        return iter(sorted(self.lookup_dict.values(), key=attrgetter("__name__")))
+    def extract_synonyms(self, element: Type[X]) -> Collection[str]:
+        """Get synonyms from an element."""
+        if not self.synonyms_attribute:
+            return []
+        return getattr(element, self.synonyms_attribute, None) or []
 
     @classmethod
     def from_subclasses(
         cls, base: Type[X], *, skip: Optional[Collection[Type[X]]] = None, **kwargs
-    ) -> "Resolver":
+    ) -> "ClassResolver":
         """Make a resolver from the subclasses of a given class.
 
         :param base: The base class whose subclasses will be indexed
@@ -211,25 +141,12 @@ class Resolver(Generic[X]):
         :return: A resolver instance
         """
         skip = set(skip) if skip else set()
-        return Resolver(
+        return cls(
             {subcls for subcls in get_subclasses(base) if subcls not in skip},
             base=base,
             **kwargs,
         )
 
-    @classmethod
-    def from_entrypoint(cls, group: str, *, base: Type[X], **kwargs) -> "Resolver":
-        """Make a resolver from the classes registered at the given entrypoint."""
-        classes = set()
-        for entry in iter_entry_points(group=group):
-            try:
-                subcls = entry.load()
-            except ImportError:
-                logger.warning("could not load %s", entry.name)
-            else:
-                classes.add(subcls)
-        return Resolver(classes=classes, base=base, **kwargs)
-
     def normalize_inst(self, x: X) -> str:
         """Normalize the class name of the instance."""
         return self.normalize_cls(x.__class__)
@@ -238,10 +155,6 @@ class Resolver(Generic[X]):
         """Normalize the class name."""
         return self.normalize(cls.__name__)
 
-    def normalize(self, s: str) -> str:
-        """Normalize the string with this resolve's suffix."""
-        return normalize_string(s, suffix=self.suffix)
-
     def lookup(self, query: HintOrType[X], default: Optional[Type[X]] = None) -> Type[X]:
         """Lookup a class."""
         return get_cls(
@@ -283,17 +196,6 @@ class Resolver(Generic[X]):
         # An instance was passed, and it will go through without modification.
         return query
 
-    def make_safe(
-        self,
-        query: HintOrType[X],
-        pos_kwargs: Optional[Mapping[str, Any]] = None,
-        **kwargs,
-    ) -> Optional[X]:
-        """Run make, but pass through a none query."""
-        if query is None:
-            return None
-        return self.make(query=query, pos_kwargs=pos_kwargs, **kwargs)
-
     def make_from_kwargs(
         self,
         data: Mapping[str, Any],
@@ -316,43 +218,6 @@ class Resolver(Generic[X]):
         pos_kwargs = data.get(f"{key}_{kwargs_suffix}", {})
         return self.make(query=query, pos_kwargs=pos_kwargs, **o_kwargs)
 
-    def get_option(
-        self,
-        *flags: str,
-        default: Hint[Type[X]] = None,
-        as_string: bool = False,
-        **kwargs,
-    ):
-        """Get a click option for this resolver."""
-        if default is None:
-            if self.default is None:
-                raise ValueError("no default given either from resolver or explicitly")
-            default = self.default
-        else:
-            default = self.lookup(default)
-        default = self.normalize_cls(default)
-
-        import click
-
-        return click.option(
-            *flags,
-            type=click.Choice(list(self.lookup_dict), case_sensitive=False),
-            default=default,
-            show_default=True,
-            callback=None if as_string else _make_callback(self.lookup),
-            **kwargs,
-        )
-
-    @property
-    def options(self) -> Set[str]:
-        """Return the normalized option names."""
-        return set(self.lookup_dict.keys()).union(self.synonyms.keys())
-
-    @property
-    def classes(self) -> Set[Type[X]]:
-        """Return the available classes."""
-        return set(self.lookup_dict.values())
-
     def ray_tune_search_space(self, kwargs_search_space: Optional[Mapping[str, Any]] = None):
         """Return a search space for ray.tune.
 
@@ -465,19 +330,8 @@ class Resolver(Generic[X]):
         ]
 
 
-def _not_hint(x: Any) -> bool:
-    return x is not None and not isinstance(x, (str, type))
-
-
-def get_subclasses(cls: Type[X]) -> Iterable[Type[X]]:
-    """Get all subclasses.
-
-    :param cls: The ancestor class
-    :yields: Descendant classes of the ancestor class
-    """
-    for subclass in cls.__subclasses__():
-        yield from get_subclasses(subclass)
-        yield subclass
+#: An alias to ClassResolver for backwards compatibility
+Resolver = ClassResolver
 
 
 def get_cls(
@@ -511,41 +365,3 @@ def get_cls(
     elif isinstance(query, type) and issubclass(query, base):
         return query
     raise TypeError(f"Not subclass of {base.__name__}: {query}")
-
-
-def normalize_string(s: str, *, suffix: Optional[str] = None) -> str:
-    """Normalize a string for lookup."""
-    s = s.lower().replace("-", "").replace("_", "").replace(" ", "")
-    if suffix is not None and s.endswith(suffix.lower()):
-        return s[: -len(suffix)]
-    return s.strip()
-
-
-def _make_callback(f: Callable[[X], Y]) -> Callable[["click.Context", "click.Parameter", X], Y]:
-    def _callback(_ctx: "click.Context", _param: "click.Parameter", value: X) -> Y:
-        return f(value)
-
-    return _callback
-
-
-def upgrade_to_sequence(x: Union[X, Sequence[X]]) -> Sequence[X]:
-    """Ensure that the input is a sequence.
-
-    :param x: A literal or sequence of literals (don't consider a string x as a sequence)
-    :return: If a literal was given, a one element tuple with it in it. Otherwise, return the given value.
-
-    >>> upgrade_to_sequence(1)
-    (1,)
-    >>> upgrade_to_sequence((1, 2, 3))
-    (1, 2, 3)
-    >>> upgrade_to_sequence("test")
-    ('test',)
-    >>> upgrade_to_sequence(tuple("test"))
-    ('t', 'e', 's', 't')
-    """
-    if isinstance(x, str):
-        return (x,)  # type: ignore
-    elif isinstance(x, collections.abc.Sequence):
-        return x
-    else:
-        return (x,)
